--[[
	Rummage: Chat-based search for items across alts.

	Inspired by MailTo, by Vincent of Blackhand
        Bima, aka Tale, aka David C Lawrence <reg+wow-addons@dd.org>

        Issue: can't call CharacterProfiler update functions
	Issue: expired mail is reported
	Issue: CharacterProfiler has a bug with saving the keyring
        Issue: mailbox data doesn't have item links
	Feature request: search other realms
        Feature request: count money (MoneyFu does, but not money in mailboxes)
]]--

Rummage = AceLibrary("AceAddon-2.0"):new("AceConsole-2.0")
local L = AceLibrary("AceLocale-2.2"):new("Rummage")

local options = {
    name = L['search'],
    type = 'text',
    desc = L['Search for an item on self and alts'],
    usage = L['<item name to search for>'],
    get = false,
    set = "rummage",
}

Rummage.version = "2.0."..string.sub("$Revision: 25931 $", 12, -3)
Rummage.date = string.sub("$Date: 2007-01-23 18:43:23 -0500 (Tue, 23 Jan 2007) $", 8, 17)
Rummage:RegisterChatCommand({'/rummage', '/rum', '/' .. L['find']}, options)

local function check(where, lookingfor, list, total)
    for slot, item in pairs(where) do
        -- For mailbox items, the actual item is down one more table.
        if item.Sender and item.Item then
            item = item.Item
        end
        if item.Name and strlower(item.Name):find(lookingfor, 1, true) then
	  -- mailbox items do not have access to item IDs.
          local key = item.Item or item.Name
	  -- Strip the unique ID off if key is an item ID
          key = string.gsub(key, '^(.+):[1-9]%d+$', '%1:0')
          found = list[key] or {}
          found.Name = item.Name
	  found.Quantity = (found.Quantity or 0) + (item.Quantity or 1)
          -- Add back the alpha if CharacterProfiler stripped it off.
	  local color = string.gsub(item.Color, '^(......)$', 'ff%1')
          found.Color = color
          list[key] = found

          newtotal = total[item.Name] or {}
          newtotal.Name = item.Name
          newtotal.Quantity = (newtotal.Quantity or 0) + (item.Quantity or 1)
          newtotal.Color = color
          total[item.Name] = newtotal
        end
    end
    return list
end

local function report(player, where, list)
    local txt = ''
    for item, attr in pairs(list) do
        if txt ~= '' then txt = txt .. ', ' end

        txt = txt .. '|c' .. attr.Color
        if (item == attr.Name) then
            txt = txt .. item
        else
            txt = txt .. format('|Hitem:%s|h[%s]|h', item, attr.Name, item)
        end
        txt = txt .. '|r'
        if (attr.Quantity > 1) then
            txt = txt .. format('|cffffff99(x%d)|r', attr.Quantity)
        end
    end
    if txt ~= '' then 
        Rummage:Print(format("%s%s%s(%s)|r: %s", 
                      '|cff00ffff', player, '|cffccff66', where, txt))
    end
end

function Rummage:rummage(item)
    -- Parse out the name of an item from a link.
    local _, _, _, name = strfind(item, 'item:(%d+):.+%[(.+)%]')
    if name then item = name end

    item = strlower(item)

    -- force CharacterProfiler update
    -- this won't update the Bank if you are at the bank
    -- the rpgo_Get* functions all use a variable that
    -- is local to CharacterProfiler
    -- rpgoCP_UpdateProfile()

    local CP = myProfile[GetCVar('realmName')]['Character']
    table.sort(CP)

    total = {}
    self:Print(format('|cffffff99%s|r "%s"',
	              L['looking for items matching'], item))
    for player, data in pairs(CP) do
	list = {}
	list = check(data.Equipment, item, list, total)
	report(player, L["Worn"], list)

        list = {}
        for bagname, bag in pairs(data.Inventory) do
            -- CharacterProfiler currently has a data structure 
	    -- inconsistency with the way the keyring is saved.
	    -- This will skip scanning the keyring, but as near
	    -- as I can tell, there isn't any item information
	    -- being saved there anyway.
            if (bag.Contents and bagname ~= "Bag5") then 
                list = check(bag.Contents, item, list, total)
            end
        end
        report(player, L['Inventory'], list)

        if data.Bank then
            list = {}
            for _, bag in pairs(data.Bank) do
                if (bag.Contents) then
                    check(bag.Contents, item, list, total)
                end
            end
            report(player, L['Bank'], list)
        end

        if data.MailBox then
            list = {}
	    list = check(data.MailBox, item, list, total)
	    report(player, L['Mail'], list)
        end
    end
    report(L['TOTAL'], L['all slots'], total)
end
